import re
from disnake import Embed


def get_user_id(field_index: int, embed: Embed) -> int:
    """Gets the user ID from the specified field index in the given embed.
    Args:
        field_index (int): Field index.
        embed (Embed): Embed object.
    Returns:
        int: DiscordID
    """
    user_id_str = re.search(r"<@!?(\d+)>", embed.fields[field_index].value).group(1)
    return int(user_id_str)



import sys
import traceback

from loguru import logger


class CustomLogger:
    _logger = logger
    _logger.remove()

    _logger.add(
        sys.stderr,
        format="<blue>[{time:DD.MM.YY / HH:mm:ss}]</blue> <green>({level})</green> <green>{message}</green>",
        colorize=True,
        level="INFO",
        filter=lambda record: record["level"].name == "INFO",
    )
    _logger.add(
        sys.stderr,
        format="<blue>[{time:DD.MM.YY / HH:mm:ss}]</blue> <yellow>({level})</yellow> <yellow>{message}</yellow>",
        colorize=True,
        level="WARNING",
        filter=lambda record: record["level"].name == "WARNING",
    )
    _logger.add(
        sys.stderr,
        format=(
            "<blue>[{time:DD.MM.YY / HH:mm:ss}]</blue> <red>({level})</red> <red>{message}</red>\n"
            "<red>╰─> {extra[traceback]}</red>"
        ),
        colorize=True,
        level="ERROR",
        filter=lambda record: record["level"].name == "ERROR",
    )

    @classmethod
    def info(cls, message, module: str = None):
        log_message = f"[{module}] {message}" if module else str(message)
        cls._logger.info(log_message)

    @classmethod
    def warning(cls, message, module: str = None):
        log_message = f"[{module}] {message}" if module else str(message)
        cls._logger.warning(log_message)

    @classmethod
    def error(cls, message, module: str = None, error: Exception = None):
        if error:
            tb = traceback.format_exc()
            log_message = f"[{module}] {message}" if module else str(message)
            cls._logger.bind(traceback=tb).error(log_message)
        else:
            log_message = f"[{module}] {message}" if module else str(message)
            cls._logger.error(log_message)

log = CustomLogger()



from tortoise import Tortoise
from crxsnake.logger import log


class Database:
    DEFAULT_URL = "sqlite://settings/database/database.db"
    DEFAULT_MODELS = "src.utils"

    def __init__(self, db_url: str = None, models: str = None):
        self.db_url = db_url or self.DEFAULT_URL
        self.models = {"models": [models or self.DEFAULT_MODELS]}

    async def db_connect(self) -> None:
        """Connect to the database."""
        try:
            await Tortoise.init(db_url=self.db_url, modules=self.models)
            await Tortoise.generate_schemas()
            log.info("SqLite connected", "Database")

        except Exception as e:
            log.error("Failed to connect SqLite", "Database", e)

    @staticmethod
    async def disconnect_db() -> None:
        """Disconnect from the database."""
        await Tortoise.close_connections()



from aiohttp import ClientSession, ClientTimeout, ClientError
from crxsnake.logger import log


class Request:
    TIMEOUT = ClientTimeout(total=20)
    STEAM_USER_AGENT = "python-steam/1.4.4"

    def __init__(self, user_agent: str = None):
        self.session = None
        self.user_agent = {"User-Agent": user_agent or self.STEAM_USER_AGENT}

    async def __aenter__(self):
        self.session = ClientSession(headers=self.user_agent, timeout=self.TIMEOUT)
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.session:
            await self.session.close()

    async def request(self, url: str, method: str = "GET", **kwargs) -> str | None:
        """Make a request to a URL.
        Args:
            url (str): URL to request
            method (str, optional): Request method. Defaults to "GET".
        Returns:
            str | None: Response text or None if an error occurred
        """
        try:
            async with self.session.request(method, url, **kwargs) as response:
                response.raise_for_status()
                return await response.text()

        except ClientError as e:
            log.error("Error when requesting", "Request", e)
        except Exception as e:
            log.error(f"Unexpected mistake", "Request", e)
        return None



from pathlib import Path
from disnake.ext.commands import Bot
from crxsnake.logger import log


async def load_cogs(bot: Bot, folder: str = "src", utils: str = "utils"):
    """Loads cogs (extensions) for the bot from the specified folder.
    Args:
        bot (Bot): The bot instance.
        folder (str, optional): The main folder containing the cogs. Defaults to "src".
        utils (str, optional): The name of the utilities folder to exclude. Defaults to "utils".

    Raises:
        Exception: If an error occurs while loading cogs.
    """
    try:
        base_path = Path.cwd() / folder

        for entry in base_path.iterdir():
            if entry.is_dir() and entry.name != utils:
                for filename in entry.glob("*.py"):
                    bot.load_extension(f"{folder}.{entry.name}.{filename.stem}")

        log.info("Cogs successfully loaded", "Discord")

    except Exception as e:
        log.error(f"An error occurred while loading modules", "Discord", e)



from typing import Optional, Tuple, List
from disnake import TextChannel, User, Embed, Forbidden, HTTPException
from crxsnake.logger import log


class EmbedLog:

    @classmethod
    def build_embed(
        cls,
        title: Optional[str] = None,
        color: Optional[int] = None,
        fields: Optional[List[Tuple[str, str, bool]]] = None,
        desc: Optional[str] = None,
        url: Optional[str] = None,
        thumbnail: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None,
    ) -> Embed:

        embed = Embed(title=title, description=desc, color=color, url=url)
        if fields:
            for name, value, inline in fields:
                embed.add_field(name=name, value=value, inline=inline)

        if thumbnail:
            embed.set_thumbnail(url=thumbnail)
        if image:
            embed.set_image(url=image)
        if footer:
            embed.set_footer(text=footer)

        return embed

    async def send_channel(
        self,
        channel: TextChannel,
        title: Optional[str] = None,
        color: Optional[int] = None,
        content: Optional[str] = None,
        fields: Optional[List[Tuple[str, str, bool]]] = None,
        desc: Optional[str] = None,
        url: Optional[str] = None,
        thumbnail: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None
    ) -> None:

        try:
            embed = self.build_embed(title, color, fields, desc, url, thumbnail, image, footer)
            await channel.send(content=content, embed=embed)
        except Forbidden:
            log.error("Not enough permissions to send embed", "LogEmbed")
        except HTTPException as e:
            log.error("Error when sending embed", "LogEmbed", e)

    async def send_user(
        self,
        user: User,
        title: Optional[str] = None,
        color: Optional[int] = None,
        content: Optional[str] = None,
        fields: Optional[List[Tuple[str, str, bool]]] = None,
        desc: Optional[str] = None,
        url: Optional[str] = None,
        thumbnail: Optional[str] = None,
        image: Optional[str] = None,
        footer: Optional[str] = None
    ) -> None:
        try:
            embed = self.build_embed(title, color, fields, desc, url, thumbnail, image, footer)
            await user.send(content=content, embed=embed)
        except Exception:
            pass



# Main file for the crxsnake package
from .database import Database
from .readers import read_json, write_json, read_jsonc
from .logger import log

# Main utils for the crxsnake package
from .utils.steam import get_steam_id
from .utils.misc import restart

# Discord utils for the crxsnake package
from .utils.discord.cogs import load_cogs
from .utils.discord.colors import Colors
from .utils.discord.embed import EmbedLog
from .utils.discord.parse import get_user_id

# DayZ utils for the crxsnake package
from .utils.dayz.convert import steam_to_be_guid, steam_to_dayz_guid
from .utils.dayz.hotline import Hotline


__all__ = [
    "Database",
    "read_json",
    "write_json",
    "read_jsonc",
    "log",

    "get_steam_id",
    "restart",

    "load_cogs",
    "Colors",
    "EmbedLog",
    "get_user_id",

    "steam_to_be_guid",
    "steam_to_dayz_guid",
    "Hotline",
]



import re
import json

from aiohttp import ClientError

from crxsnake.web.request import Request
from crxsnake.logger import log


async def get_steam_id(steam_profile_url: str) -> int | None:
    """Get steam id from steam profile url.

    Args:
        steam_profile_url (str)
    Returns:
        int | None
    """
    async with Request() as steam:
        try:
            response = await steam.request(steam_profile_url)
            if not response:
                return None

            data_match = re.search(r"g_rgProfileData\s*=\s*(?P<json>{.*?});", response)
            if data_match:
                return int(json.loads(data_match.group("json"))["steamid"])

        except ClientError as e:
            log.error("Error during request", "Steam", e)
        except (KeyError, ValueError, json.JSONDecodeError) as e:
            log.error("Error during data processing", "Steam", e)

    return None
